// GeneticMonsters.cpp - DirectInput keyboard demo

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  
#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <objbase.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <direct.h>
#include <wchar.h>
#include <fcntl.h>

#include <ddraw.h>     // directX includes
#include <dinput.h> 
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>

#include "t3dlib3.h"
#include "gpdumb1.h"
#include "resource.h"


// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINXCLASS"  // class name

#define WINDOW_WIDTH    64   // size of window
#define WINDOW_HEIGHT   48

// skelaton directions
#define SKELATON_EAST         0
#define SKELATON_NEAST        1  
#define SKELATON_NORTH        2
#define SKELATON_NWEST        3
#define SKELATON_WEST         4
#define SKELATON_SWEST        5
#define SKELATON_SOUTH        6
#define SKELATON_SEAST        7


// weapons defines
#define MAX_PLASMA         8   // max number of plasma pulses
#define PLASMA_STATE_OFF   0   // this plasma is dead or off
#define PLASMA_STATE_ON    1   // this one is alive and in flight

#define PLASMA_ANIM_PLAYER     0  // a plasma torpedeo from player

// sound id's
#define MAX_FIRE_SOUNDS       8
#define MAX_EXPL_SOUNDS       8

// asteroid field defines
#define MAX_ROCKS          8  // 10 monsters show at one time on the screen
#define MAX_ROCKS2         20 // 40 monsters show at one time on the screen
#define MAX_ROCKS3         70 // 80 monsters show at one time on the screen
#define ROCK_STATE_OFF     0   // this rock is dead or off
#define ROCK_STATE_ON      1   // this one is alive and in flight

#define ROCK_LARGE         0   // sizes of rock
#define ROCK_MEDIUM        1
#define ROCK_SMALL         2

// powerups field defines
#define MAX_POWERUPS		  5   // 5 powerups show at one time on the screen
#define POWERUP_STATE_OFF     0   // this powerup is dead or off
#define POWERUP_STATE_ON      1   // this powerup is alive and in flight


// explosion defines 
#define MAX_BURSTS          8
#define BURST_STATE_OFF     0   // this burst is dead or off
#define BURST_STATE_ON      1   // this one is alive

// defines for player states
#define PLAYER_STATE_DEAD     0
#define PLAYER_STATE_DYING    1
#define PLAYER_STATE_ALIVE    2 

// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL);
int Game_Shutdown(void *parms=NULL);
int Game_Main(void *parms=NULL);
//int Switch_Level(void *parms=NULL);
int Switch_Level(int);

// GLOBALS ////////////////////////////////////////////////

HWND main_window_handle = NULL; // save the window handle
HINSTANCE main_instance = NULL; // save the instance
char buffer[80];                // used to print text
int screen_seq;					// screen sequence on a level
int plasma_x=0;					// direction of plasma (x)
int plasma_y=-16;				// direction of plasma (y)
int player_hit = 0;			// The strength of the can man
int player_limit = 600;		// The limit of the strength
int Monsters_Alive_Nbr = 20;	// The number of alive monsters to begin with
							// SHOULD BE 20
int Monsters_Dead_Nbr = 0;		// The number of dead monsters
int Player_Health = 500;		// The health of the player to begin with
int Player_Energy = 500;		// The energy of the player to begin with
int Energy_Flag = 0;			// Tracks energy gain by the player
int Touch_Flag = 0;	
int Level2_Flag = 0;			// Player killed 20 monsters
int Level3_Flag = 0;			// Player killed 60 monsters

// directinput globals
LPDIRECTINPUT        lpdi      = NULL;    // dinput object
LPDIRECTINPUTDEVICE  lpdikey   = NULL;    // dinput keyboard
LPDIRECTINPUTDEVICE  lpdimouse = NULL;    // dinput mouse
LPDIRECTINPUTDEVICE  lpdijoy   = NULL;    // dinput joystick 
LPDIRECTINPUTDEVICE2 lpdijoy2  = NULL;    // dinput joystick
GUID                 joystickGUID;        // guid for main joystick
char                 joyname[80];         // name of joystick

// these contain the target records for all di input packets
UCHAR keyboard_state[256]; // contains keyboard state table
DIMOUSESTATE mouse_state;  // contains state of mouse
DIJOYSTATE joy_state;      // contains state of joystick

// demo globals
BOB          skelaton;     // the player skelaton

BOB plasma[MAX_PLASMA];    // plasma pulses
BOB rocks[MAX_ROCKS];      // the asteroids
BOB rocks2[MAX_ROCKS2];      // the asteroids
BOB rocks3[MAX_ROCKS3];      // the asteroids
BOB bursts[MAX_BURSTS];    // the explosion bursts
BOB powerups[MAX_POWERUPS];// the number of powerups

BITMAP_IMAGE background_bmp; // used to hold backgrounds

// set the sizes of the monsters.  There are three sizes.  Baby monsters newly born 
// will be of 64 size
int rock_sizes[3] = {256,128,64}; // width X height sizes for scaler

// player state variables
int player_state = PLAYER_STATE_ALIVE;
int player_score = 0;  // the score
int player_ships = 5;  // ships left
int player_damage = 0; // damage of player

// animation sequences for bob
int skelaton_anims[8][4] = { {0,1,0,2},
                             {0+4,1+4,0+4,2+4},
                             {0+8,1+8,0+8,2+8},
                             {0+12,1+12,0+12,2+12},
                             {0+16,1+16,0+16,2+16},
                             {0+20,1+20,0+20,2+20},
                             {0+24,1+24,0+24,2+24},
                             {0+28,1+28,0+28,2+28}, };


BITMAP_IMAGE reactor;      // the background  

// sound id's
int intro_music_id  = -1,
    main_music_id   = -1,
    ready_id        = -1,
    engines_id      = -1,
    scream_id       = -1,
    game_over_id    = -1,
    mine_powerup_id = -1,
    deactivate_id   = -1,
    main_menu_id    = -1,
    beep0_id             = -1,
    beep1_id             = -1,
    station_blow_id      = -1,
	correct_id			 = -1,
	applause_id			 = -1,
    station_throb_id     = -1;

int expl_ids[MAX_EXPL_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};
int fire_ids[MAX_FIRE_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};
 

// helper functions for game logic
void Draw_Info(void);
void Start_Burst(int x, int y, int width, int height, int xv,int yv);
void Draw_Bursts(void);
void Move_Bursts(void);
void Delete_Bursts(void);
void Init_Bursts(void);

void Draw_Rocks(void);
void Start_Rock(int x, int y, int size,int xv, int yv);
void Move_Rocks(void);
void Delete_Rocks(void);
void Init_Rocks(void);

void Draw_Rocks2(void);
void Start_Rock2(int x, int y, int size,int xv, int yv);
void Move_Rocks2(void);
void Delete_Rocks2(void);
void Init_Rocks2(void);

void Draw_Rocks3(void);
void Start_Rock3(int x, int y, int size,int xv, int yv);
void Move_Rocks3(void);
void Delete_Rocks3(void);
void Init_Rocks3(void);


void Draw_Powerups(void);
void Start_Powerup(int x, int y, int size,int xv, int yv);
void Move_Powerups(void);
void Delete_Powerups(void);
void Init_Powerups(void);

//void Fire_Plasma(int x,int y, int velx, int vely);
void Fire_Plasma(int x,int y, int velx, int vely, int source);
void Draw_Plasma(void);
void Move_Plasma(void);
int Collision_Test(int x1, int y1, int w1, int h1, 
                   int x2, int y2, int w2, int h2);

void Delete_Plasma(void);
void Init_Plasma(void);
void Do_Intro(void);
int Load_Sound_Music(void);

// PROTOTYPES //////////////////////////////////////////////

int Color_Scan(int x1, int y1, int x2, int y2, 
      UCHAR scan_color, UCHAR *scan_buffer, int scan_lpitch);

// FUNCTIONS //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT	ps;		   // used in WM_PAINT
HDC			hdc;	   // handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
		return(0);
		} break;

    case WM_PAINT:
         {
         // start painting
         hdc = BeginPaint(hwnd,&ps);

         // end painting
         EndPaint(hwnd,&ps);
         return(0);
        } break;

	case WM_DESTROY: 
		{
		// kill the application			
		PostQuitMessage(0);
		return(0);
		} break;
	
	case WM_COMMAND:
         {
          switch(LOWORD(wparam))
                {
                // handle the FILE menu
                case IDM_EXIT:
                     {
                     // terminate window
                     PostQuitMessage(0);
                     } break;

                // handle the HELP menu
                case IDM_NEWGAME:                 
                     {
                     //  pop up a message box
                     MessageBox(hwnd, "Menu MIDI Demo", 
                               "About MIDI Menu",
                                MB_OK | MB_ICONEXCLAMATION);
                     } break;
                default: break;
             } // end switch wparam
          } break; // end WM_COMMAND

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{
// this is the winmain function

WNDCLASS winclass;	// this will hold the class we create
HWND	 hwnd;		// generic window handle
MSG		 msg;		// generic message
HDC      hdc;       // generic dc
PAINTSTRUCT ps;     // generic paintstruct

// first fill in the window class stucture
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= "IDR_MENU1"; 
winclass.lpszClassName	= WINDOW_CLASS_NAME;

// register the window class
if (!RegisterClass(&winclass))
	return(0);

// create the window, note the use of WS_POPUP
if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
						  "WinX Game Console",	 // title
						   WS_POPUP | WS_VISIBLE | WS_SYSMENU | WS_CAPTION | WS_MAXIMIZE,
					 	  0,0,	   // x,y
						  400,  // width
                          300, // height
						  NULL,	   // handle to parent 
						  LoadMenu(hinstance, "IDR_MENU1"),   // handle to menu
						  hinstance,// instance
						  NULL)))	// creation parms
return(0);

// save the window handle and instance in a global
main_window_handle = hwnd;
main_instance      = hinstance;

int first = 0;

// perform all game console specific initialization
Game_Init();


// enter main event loop
while(1)
	{
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
		// test if this is a quit
        if (msg.message == WM_QUIT)
           break;
	
		// translate any accelerator keys
		TranslateMessage(&msg);

		// send the message to the window proc
		DispatchMessage(&msg);
		} // end if

    // main game processing goes here
    Game_Main();

	} // end while

// shutdown game and release all resources
Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

///////////////////////////////////////////////////////////

int Color_Scan(int x1, int y1, int x2, int y2, 
               UCHAR scan_color, UCHAR *scan_buffer, int scan_lpitch)
{
// this function implements a crude collision technique
// based on scanning for a specific color within a rectangle

// clip rectangle

// x coords first    
if (x1 >= SCREEN_WIDTH)
   x1=SCREEN_WIDTH-1;
else
if (x1 < 0)
   x1=0;

if (x2 >= SCREEN_WIDTH)
   x2=SCREEN_WIDTH-1;
else
if (x2 < 0)
   x2=0;

// now y-coords
if (y1 >= SCREEN_HEIGHT)
   y1=SCREEN_HEIGHT-1;
else
if (y1 < 0)
   y1=0;

if (y2 >= SCREEN_HEIGHT)
   y2=SCREEN_HEIGHT-1;
else
if (y2 < 0)
   y2=0;

// scan the region
scan_buffer +=y1*scan_lpitch;

for (int scan_y=y1; scan_y<=y2; scan_y++)
    {
    for (int scan_x=x1; scan_x<=x2; scan_x++)
        {
        if (scan_buffer[scan_x] == scan_color)
            return(1);
        } // end for x

    // move down a line
    scan_buffer+=scan_lpitch;

    } // end for y

// return failure
return(0);

} // end Color_Scan

// WINX GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms)
{
// this function is where you do all the initialization 
// for your game

int index;         // looping var
char filename[80]; // used to build up files names

// initialize directdraw
DD_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

// keyboard creation section ////////////////////////////////

// first create the direct input object
if (DirectInputCreate(main_instance,DIRECTINPUT_VERSION,&lpdi,NULL)!=DI_OK)
   return(0);

// create a keyboard device  //////////////////////////////////
if (lpdi->CreateDevice(GUID_SysKeyboard, &lpdikey, NULL)!=DI_OK)
   return(0);

// set cooperation level
if (lpdikey->SetCooperativeLevel(main_window_handle, 
            DISCL_NONEXCLUSIVE | DISCL_BACKGROUND)!=DI_OK)
 return(0);

// set data format
if (lpdikey->SetDataFormat(&c_dfDIKeyboard)!=DI_OK)
   return(0);

// acquire the keyboard
if (lpdikey->Acquire()!=DI_OK)
   return(0);

// hide the mouse
ShowCursor(FALSE);

// initialize directsound
DSound_Init();

// load all the sound and music
Load_Sound_Music();

///////////////////////////////////////////////////////////

// load the background for the beginning of the game
Load_Bitmap_File(&bitmap8bit, "BackMain.bmp");

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// create and load the reactor bitmap image
Create_Bitmap(&reactor, 0,0, 640, 480);
Load_Image_Bitmap(&reactor,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// now let's load in all the frames for the skelaton!!!
// parameters: 
// ptr to bob to create
// initial position of bob
// size of bob (width, height)
// total number of frames for bob
// attributes for bob
// surface memory flags, 0 is VRAM

// create skelaton bob
if (!Create_BOB(&skelaton,0,0,56,70,32,
           BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM,DDSCAPS_SYSTEMMEMORY))
   return(0);

// load the frames in 8 directions, 4 frames each
// each set of frames has a walk and a fire, frame sets
// are loaded in counter clockwise order looking down
// from a birds eys view or the x-z plane
for (int direction = 0; direction < 8; direction++)
    { 
    // build up file name
    sprintf(filename,"SKELSP%d.BMP",direction);

    // load in new bitmap file
    Load_Bitmap_File(&bitmap8bit,filename);
 
    Load_Frame_BOB(&skelaton,&bitmap8bit,0+direction*4,0,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&skelaton,&bitmap8bit,1+direction*4,1,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&skelaton,&bitmap8bit,2+direction*4,2,0,BITMAP_EXTRACT_MODE_CELL);  
    Load_Frame_BOB(&skelaton,&bitmap8bit,3+direction*4,0,1,BITMAP_EXTRACT_MODE_CELL);  

    // unload the bitmap file
    Unload_Bitmap_File(&bitmap8bit);

    // set the animation sequences for skelaton
    Load_Animation_BOB(&skelaton,direction,4,skelaton_anims[direction]);

    } // end for direction

// set up stating state of skelaton
Set_Animation_BOB(&skelaton, 0);
Set_Anim_Speed_BOB(&skelaton, 4);
Set_Vel_BOB(&skelaton, 0,0);
Set_Pos_BOB(&skelaton, 0, 128);

// set clipping rectangle to screen extents so mouse cursor
// doens't mess up at edges
RECT screen_rect = {0,0,screen_width,screen_height};
lpddclipper = DD_Attach_Clipper(lpddsback,1,&screen_rect);

// initialize the plasma pulses
Init_Plasma();

// init rocks
Init_Rocks();

// init powerups
Init_Powerups();

// init all the explosions
Init_Bursts();

// hide the mouse
ShowCursor(FALSE);

// start the intro music
DSound_Play(intro_music_id);

// Runs the Intro
Do_Intro();

// Puts the character at the bottom of the screen
skelaton.y = 400;
skelaton.x = 200;

// First screen on the level
screen_seq = 1;

//Sleep(10000);
// return success
return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

void Init_Plasma(void)
{
// this function initializes and loads all the plasma 
// weapon pulses

// load the plasma imagery 
Load_Bitmap_File(&bitmap8bit, "Bullet3.BMP");


#if 1
// now create and load each plasma pulse
for (int pulse=0; pulse<MAX_PLASMA; pulse++)
    {
    // create the bob to hold pulse
    Create_BOB(&plasma[pulse],0,0,32,32,6,
               BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
               DDSCAPS_SYSTEMMEMORY);
 
    // load animation frames
    for (int frame=0; frame<6; frame++)
         Load_Frame_BOB(&plasma[pulse],&bitmap8bit,frame,frame,0,BITMAP_EXTRACT_MODE_CELL);  

    // set animation rate
    Set_Anim_Speed_BOB(&plasma[pulse],1);

    // set state to off
    plasma[pulse].state = PLASMA_STATE_OFF;

    } // end for pulse
#endif

#if 0
// create the first bob
Create_BOB(&plasma[0],0,0,32,32,6,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame<6; frame++)
     Load_Frame_BOB(&plasma[0],&bitmap8bit,frame,frame,0,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
Set_Anim_Speed_BOB(&plasma[0],1);

// set state to off
plasma[0].state = PLASMA_STATE_OFF;

for (int pulse=1; pulse<MAX_PLASMA; pulse++)
    {
    memcpy(&plasma[pulse], &plasma[0], sizeof(BOB));
    } // end for pulse

#endif


// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Plasma

///////////////////////////////////////////////////////////

void Delete_Plasma(void)
{
// this function simply deletes all memory and surfaces
// related to the plasma pulses

for (int index=0; index<MAX_PLASMA; index++)
    Destroy_BOB(&plasma[index]);

} // end Delete_Plasma

///////////////////////////////////////////////////////////

int Collision_Test(int x1, int y1, int w1, int h1, 
                   int x2, int y2, int w2, int h2) 
{
// this function tests if the two rects overlap

// get the radi of each rect
int width1  = (w1>>1) - (w1>>3);
int height1 = (h1>>1) - (h1>>3);

int width2  = (w2>>1) - (w2>>3);
int height2 = (h2>>1) - (h2>>3);

// compute center of each rect
int cx1 = x1 + width1;
int cy1 = y1 + height1;

int cx2 = x2 + width2;
int cy2 = y2 + height2;

// compute deltas
int dx = abs(cx2 - cx1);
int dy = abs(cy2 - cy1);

// test if rects overlap
if (dx < (width1+width2) && dy < (height1+height2))
   return(1);
else
// else no collision
return(0);

} // end Collision_Test

///////////////////////////////////////////////////////////

void Move_Plasma(void)
{
// this function moves all the plasma pulses and checks for
// collision with the rocks

for (int index=0; index<MAX_PLASMA; index++)
    {
    // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_ON)
        {
        // move the pulse upward
        Move_BOB(&plasma[index]);
  
        // test for boundaries
        if (plasma[index].y < 0 || plasma[index].y > 500 
			|| plasma[index].x < 0 || plasma[index].x > 700)
            {
            // kill the pulse
            plasma[index].state = PLASMA_STATE_OFF;

            // move to next pulse
            continue;
            } // end if

        // test for collision with rocks
		// collision of plasma with any of the asteroids which are monsters in 
		// this case cause the death of the monster in touch
        if (Level2_Flag == 0)
		{
		for (int rock=0; rock<MAX_ROCKS; rock++)
            {			
            if (rocks[rock].state==ROCK_STATE_ON)
                {
                // test for collision 
                if (Collision_Test(plasma[index].x, plasma[index].y, 
                                   plasma[index].width, plasma[index].height,
                                   rocks[rock].x, rocks[rock].y,
                                   rocks[rock].varsI[1], rocks[rock].varsI[1]))
                    {
                    // kill pulse
                    plasma[index].state = PLASMA_STATE_OFF;
  
                    // start explosion
                    Start_Burst(plasma[index].x, plasma[index].y, 
                               42,36,
                               rocks[rock].xv>>1, rocks[rock].yv>>1);

                        // kill rock
                        rocks[rock].state = ROCK_STATE_OFF;

						// decrease the number of Alive Monsters 
						Monsters_Alive_Nbr = Monsters_Alive_Nbr - 1;
					
						// increase the number of Dead Monsters
						Monsters_Dead_Nbr = Monsters_Dead_Nbr + 1;

                    // break out of loop
                    break;

                   } // end if collision
                } // end if rock alive
            } // end for rock
		}
		else if ((Level2_Flag == 1) && (Level3_Flag == 0))
		{
		for (int rock=0; rock<MAX_ROCKS2; rock++)
            {			
            if (rocks2[rock].state==ROCK_STATE_ON)
                {
                // test for collision 
                if (Collision_Test(plasma[index].x, plasma[index].y, 
                                   plasma[index].width, plasma[index].height,
                                   rocks2[rock].x, rocks2[rock].y,
                                   rocks2[rock].varsI[1], rocks2[rock].varsI[1]))
                    {
                    // kill pulse
                    plasma[index].state = PLASMA_STATE_OFF;
  
                    // start explosion
                    Start_Burst(plasma[index].x, plasma[index].y, 
                               42,36,
                               rocks2[rock].xv>>1, rocks2[rock].yv>>1);

                    // kill rock
                    rocks2[rock].state = ROCK_STATE_OFF;

					// decrease the number of Alive Monsters 
					Monsters_Alive_Nbr = Monsters_Alive_Nbr - 1;
					
					// increase the number of Dead Monsters
					Monsters_Dead_Nbr = Monsters_Dead_Nbr + 1;

                    // break out of loop
                    break;

                   } // end if collision
                } // end if rock alive
            } // end for rock
		}//end else if
		else if (Level3_Flag == 1)
		{
		for (int rock=0; rock<MAX_ROCKS3; rock++)
            {			
            if (rocks3[rock].state==ROCK_STATE_ON)
                {
                // test for collision 
                if (Collision_Test(plasma[index].x, plasma[index].y, 
                                   plasma[index].width, plasma[index].height,
                                   rocks3[rock].x, rocks3[rock].y,
                                   rocks3[rock].varsI[1], rocks3[rock].varsI[1]))
                    {
                    // kill pulse
                    plasma[index].state = PLASMA_STATE_OFF;
  
                    // start explosion
                    Start_Burst(plasma[index].x, plasma[index].y, 
                               42,36,
                               rocks3[rock].xv>>1, rocks3[rock].yv>>1);

                    // kill rock
                    rocks3[rock].state = ROCK_STATE_OFF;

					// decrease the number of Alive Monsters 
					Monsters_Alive_Nbr = Monsters_Alive_Nbr - 1;
					
					// increase the number of Dead Monsters
					Monsters_Dead_Nbr = Monsters_Dead_Nbr + 1;

                    // break out of loop
                    break;

                   } // end if collision
                } // end if rock alive
            } // end for rock
		}//end else if
      } // end if
    } // end for index
} // end Move_Plasma

///////////////////////////////////////////////////////////

void Draw_Plasma(void)
{
// this function draws all the plasma pulses

for (int index=0; index<MAX_PLASMA; index++)
    {
    // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_ON)
        {
        // draw the pulse
        Draw_BOB(&plasma[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&plasma[index]);

        } // end if

    } // end for index

} // end Draw_Plasma

///////////////////////////////////////////////////////////

void Fire_Plasma(int x,int y, int velx, int vely, int source=PLASMA_ANIM_PLAYER)

//void Fire_Plasma(int x,int y, int velx, int vely)
{
// this function fires a plasma pulse at the given starting
// position and velocity, of course, one must be free for 
// this to work

// scan for a pulse that is available
for (int index=0; index<MAX_PLASMA; index++)
    {
    // is this one available
      // test if plasma pulse is in flight
    if (plasma[index].state == PLASMA_STATE_OFF)
		{
       // start this one up
       plasma[index].x  = x;
       plasma[index].y  = y;
       plasma[index].yv = vely;  // added to give 
	   plasma[index].xv = velx;  // plasma direction
       plasma[index].curr_frame = 0;
       plasma[index].state =  PLASMA_STATE_ON;
	   //plasma[index].counter_1 = 0; // used to track distance

	   // set animation and typing info for collision engine
	   //plasma[index].anim_state = source;
	   //Set_Animation_BOB(&plasma[index], source);

       // start sound up
       for (int sound_index=0; sound_index < MAX_FIRE_SOUNDS; sound_index++)
           {
           // test if this sound is playing
           if (DSound_Status_Sound(fire_ids[sound_index])==0)
              {
              DSound_Play(fire_ids[sound_index]);
              break;
              } // end if
        
           } // end for sound_index

       return;

       } // end if
    } // end for
} // end Fire_Plasma

///////////////////////////////////////////////////////////

void Init_Rocks(void)
{
// this function initializes and loads all the rocks 

// load the rocks imagery 
Load_Bitmap_File(&bitmap8bit, "Monsters7_U.BMP");

// create the first bob
Create_BOB(&rocks[0],0,0,96,96,20,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 20; frame++)
     Load_Frame_BOB(&rocks[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
//Set_Anim_Speed_BOB(&rocks[0],1+rand()%5);
Set_Anim_Speed_BOB(&rocks[0],1);
//Set_Vel_BOB(&rocks[0], -4+rand()%8, 4+rand()%4);
Set_Vel_BOB(&rocks[0], 1, 1);
Set_Pos_BOB(&rocks[0], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
//rocks[0].varsI[0] = rand()%3;
rocks[0].varsI[0] = rand()%3;
rocks[0].varsI[1] = rock_sizes[rocks[0].varsI[0]];

// set state to off
rocks[0].state = ROCK_STATE_OFF;

// initialize all the rocks by copying initialization from the first one
for (int rock=1; rock<MAX_ROCKS; rock++)
    {
    memcpy(&rocks[rock], &rocks[0], sizeof(BOB));

	// set animation rate
	//Set_Anim_Speed_BOB(&rocks[rock],1+rand()%5);
	Set_Anim_Speed_BOB(&rocks[rock],1);
	
	// set velocity
	//Set_Vel_BOB(&rocks[rock], -1+rand()%3, 2+rand()%2);
	Set_Vel_BOB(&rocks[rock], 0, 1);
    
    // set position
    Set_Pos_BOB(&rocks[rock], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);
    
    // set size of rock
    rocks[rock].varsI[0] = rand()%3;
    rocks[rock].varsI[1] = rock_sizes[rocks[rock].varsI[0]];
    } // end for rock

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Rocks

///////////////////////////////////////////////////////////
// Begin Level 2

///////////////////////////////////////////////////////////

void Init_Rocks2(void)
{
// this function initializes and loads all the rocks 

// load the rocks2 imagery 
Load_Bitmap_File(&bitmap8bit, "Monsters7_U.BMP");

// create the first bob
Create_BOB(&rocks2[0],0,0,96,96,20,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 20; frame++)
     Load_Frame_BOB(&rocks2[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
//Set_Anim_Speed_BOB(&rocks2[0],1+rand()%5);
Set_Anim_Speed_BOB(&rocks2[0],1);
//Set_Vel_BOB(&rocks2[0], -4+rand()%8, 4+rand()%4);
Set_Vel_BOB(&rocks2[0], 1, 1);
Set_Pos_BOB(&rocks2[0], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
//rocks2[0].varsI[0] = rand()%3;
rocks2[0].varsI[0] = rand()%3;
rocks2[0].varsI[1] = rock_sizes[rocks2[0].varsI[0]];

// set state to off
rocks2[0].state = ROCK_STATE_OFF;

// initialize all the rocks2 by copying initialization from the first one
for (int rock=1; rock<MAX_ROCKS2; rock++)
    {
    memcpy(&rocks2[rock], &rocks2[0], sizeof(BOB));

	
   
	// set animation rate
	//Set_Anim_Speed_BOB(&rocks2[rock],1+rand()%5);
	Set_Anim_Speed_BOB(&rocks2[rock],1);
	
		// set velocity
	//Set_Vel_BOB(&rocks2[rock], -1+rand()%3, 2+rand()%2);
	Set_Vel_BOB(&rocks2[rock], 0, 1);
    
    // set position
    Set_Pos_BOB(&rocks2[rock], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);
    
    // set size of rock
    rocks2[rock].varsI[0] = rand()%3;
    rocks2[rock].varsI[1] = rock_sizes[rocks2[rock].varsI[0]];
    } // end for rock

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_rocks2

///////////////////////////////////////////////////////////

void Delete_Rocks2(void)
{
// this function simply deletes all memory and surfaces
// related to the rocks2 pulses

for (int index=0; index<MAX_ROCKS2; index++)
    Destroy_BOB(&rocks2[index]);

} // end Delete_Rocks2

///////////////////////////////////////////////////////////

void Move_Rocks2(void)
{
   // this function moves all the rocks2 pulses and checks for
   // collision with the rocks2

   for (int index=0; index<MAX_ROCKS2; index++)
   {
      // test if rocks2 pulse is in flight
      if (rocks2[index].state == ROCK_STATE_ON)
      {
         // move the rock
         Move_BOB(&rocks2[index]);

         // test if rock is off screen
         if (rocks2[index].y > SCREEN_HEIGHT)
         {
            // kill rock and put back on available list
            rocks2[index].state = ROCK_STATE_OFF;
         } // end if

		// test for collision with the player
		// if the monsters touch the player, they cause damage to his health
        //if (Collision_Test(skelaton.x, skelaton.y, 
        //               skelaton.width, skelaton.height,
        //               rocks2[index].x, rocks2[index].y,
        //               rocks2[index].varsI[1], rocks2[index].varsI[1]))
	
		if (Collision_Test(skelaton.x, skelaton.y, 
                      skelaton.width, skelaton.height,
                      rocks2[index].x, rocks2[index].y,
                      rocks2[index].x, rocks2[index].y))

        {
			//player_hit++;
			rocks2[index].state = ROCK_STATE_OFF;
			Monsters_Alive_Nbr = Monsters_Alive_Nbr + 7;
			Player_Health -= 25;
		}// end if
	  } // end if
   } // end for index

// now test if it's time to add a new rock to the list
if ((rand()%6)==3)
   {
   // scan for a rock to initialize
   for (index=0; index<MAX_ROCKS2; index++)
       {
       // is this rock available?
       if (rocks2[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks2[index],1+rand()%5);
    
          // set velocity
          //Set_Vel_BOB(&rocks2[index], -4+rand()%8, 4+rand()%12);
		  Set_Vel_BOB(&rocks2[index], -1+rand()%5, 1+rand()%9);
    
          // set position
          Set_Pos_BOB(&rocks2[index], rand()%SCREEN_WIDTH, -128);
    
          // set size of rock
          rocks2[index].varsI[0] = rand()%3;
          rocks2[index].varsI[1] = rock_sizes[rocks2[index].varsI[0]];
          
          // set strength of rock
          switch(rocks2[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks2[index].varsI[2] = 100+rand()%100;
                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks2[index].varsI[2] = 40 + rand()%30;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks2[index].varsI[2] = 10;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks2[index].state = ROCK_STATE_ON;

          return;

          } // end if
       } // end for index
   } // end if
} // end Move_Rocks2

///////////////////////////////////////////////////////////

void Start_Rock2(int x, int y, int size,int xv, int yv)
{
// this functions starts a rock up with the sent parms, considering
// there is one left

// scan for a rock to initialize
   for (int index=0; index<MAX_ROCKS2; index++)
       {
       // is this rock available?
       if (rocks2[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks2[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks2[index], xv,yv);
    
          // set position
          Set_Pos_BOB(&rocks2[index], x,y);
    
          // set size of rock
          rocks2[index].varsI[0] = size;
          rocks2[index].varsI[1] = rock_sizes[rocks2[index].varsI[0]];
          
          // set strength of rock
          switch(rocks2[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks2[index].varsI[2] = 100+rand()%100;
                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks2[index].varsI[2] = 40 + rand()%30;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks2[index].varsI[2] = 10;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks2[index].state = ROCK_STATE_ON;
          
          // later
          return;

          } // end if

       } // end for index

} // end Start_Rock2

///////////////////////////////////////////////////////////

void Draw_Rocks2(void)
{
// this function draws all the rocks2 

for (int index=0; index<MAX_ROCKS2; index++)
    {
    // test if rocks2 pulse is in flight
    if (rocks2[index].state == ROCK_STATE_ON)
        {

        if (rocks2[index].varsI[0]!=ROCK_LARGE)
            {
            // draw the rock scaled
            Draw_Scaled_BOB(&rocks2[index],
                            rocks2[index].varsI[1],rocks2[index].varsI[1],
                            lpddsback);
            }
        else // draw normal
			Draw_BOB(&rocks2[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&rocks2[index]);

        } // end if

    } // end for index

} // end Draw_Rocks2

///////////////////////////////////////////////////////////

// End Level 2
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
// Begin Level 3

///////////////////////////////////////////////////////////

void Init_Rocks3(void)
{
// this function initializes and loads all the rocks 

// load the rocks3 imagery 
Load_Bitmap_File(&bitmap8bit, "Monsters7_U.BMP");

// create the first bob
Create_BOB(&rocks3[0],0,0,96,96,20,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 20; frame++)
     Load_Frame_BOB(&rocks3[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
//Set_Anim_Speed_BOB(&rocks3[0],1+rand()%5);
Set_Anim_Speed_BOB(&rocks3[0],1);
//Set_Vel_BOB(&rocks3[0], -4+rand()%8, 4+rand()%4);
Set_Vel_BOB(&rocks3[0], 1, 1);
Set_Pos_BOB(&rocks3[0], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
//rocks3[0].varsI[0] = rand()%3;
rocks3[0].varsI[0] = rand()%3;
rocks3[0].varsI[1] = rock_sizes[rocks3[0].varsI[0]];

// set state to off
rocks3[0].state = ROCK_STATE_OFF;

// initialize all the rocks3 by copying initialization from the first one
for (int rock=1; rock<MAX_ROCKS3; rock++)
    {
    memcpy(&rocks3[rock], &rocks3[0], sizeof(BOB));

	// set animation rate
	//Set_Anim_Speed_BOB(&rocks3[rock],1+rand()%5);
	Set_Anim_Speed_BOB(&rocks3[rock],1);
	
		// set velocity
	//Set_Vel_BOB(&rocks3[rock], -1+rand()%3, 2+rand()%2);
	Set_Vel_BOB(&rocks3[rock], 0, 1);
    
    // set position
    Set_Pos_BOB(&rocks3[rock], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);
    
    // set size of rock
    rocks3[rock].varsI[0] = rand()%3;
    rocks3[rock].varsI[1] = rock_sizes[rocks3[rock].varsI[0]];
    } // end for rock

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Rocks3

///////////////////////////////////////////////////////////

void Delete_Rocks3(void)
{
// this function simply deletes all memory and surfaces
// related to the rocks2 pulses

for (int index=0; index<MAX_ROCKS3; index++)
    Destroy_BOB(&rocks3[index]);

} // end Delete_Rocks3

///////////////////////////////////////////////////////////

void Move_Rocks3(void)
{
   // this function moves all the rocks2 pulses and checks for
   // collision with the rocks2

   for (int index=0; index<MAX_ROCKS3; index++)
   {
      // test if rocks2 pulse is in flight
      if (rocks3[index].state == ROCK_STATE_ON)
      {
         // move the rock
         Move_BOB(&rocks3[index]);

         // add player motion in
         //if (moving_up)
         //    rocks3[index].y+=(rocks3[index].yv>>1);

         // test if rock is off screen
         if (rocks3[index].y > SCREEN_HEIGHT)
         {
            // kill rock and put back on available list
            rocks3[index].state = ROCK_STATE_OFF;
         } // end if

		// test for collision with the player
		// if the monsters touch the player, they cause damage to his health
        //if (Collision_Test(skelaton.x, skelaton.y, 
        //               skelaton.width, skelaton.height,
        //               rocks3[index].x, rocks3[index].y,
        //               rocks3[index].varsI[1], rocks3[index].varsI[1]))
	
		if (Collision_Test(skelaton.x, skelaton.y, 
                      skelaton.width, skelaton.height,
                      rocks3[index].x, rocks3[index].y,
                      rocks3[index].x, rocks3[index].y))

        {
			//player_hit++;
			rocks3[index].state = ROCK_STATE_OFF;
			Monsters_Alive_Nbr = Monsters_Alive_Nbr + 10;
			Player_Health -= 50;
		}// end if
	  } // end if
   } // end for index

// now test if it's time to add a new rock to the list
if ((rand()%6)==3)
   {
   // scan for a rock to initialize
   for (index=0; index<MAX_ROCKS3; index++)
       {
       // is this rock available?
       if (rocks3[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks3[index],1+rand()%5);
    
          // set velocity
          //Set_Vel_BOB(&rocks3[index], -4+rand()%8, 4+rand()%12);
		  Set_Vel_BOB(&rocks3[index], -1+rand()%5, 1+rand()%9);
    
          // set position
          Set_Pos_BOB(&rocks3[index], rand()%SCREEN_WIDTH, -128);
    
          // set size of rock
          rocks3[index].varsI[0] = rand()%3;
          rocks3[index].varsI[1] = rock_sizes[rocks3[index].varsI[0]];
          
          // set strength of rock
          switch(rocks3[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks3[index].varsI[2] = 100+rand()%100;
                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks3[index].varsI[2] = 40 + rand()%30;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks3[index].varsI[2] = 10;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks3[index].state = ROCK_STATE_ON;

          return;

          } // end if
       } // end for index
   } // end if
} // end Move_Rocks3

///////////////////////////////////////////////////////////

void Start_Rock3(int x, int y, int size,int xv, int yv)
{
// this functions starts a rock up with the sent parms, considering
// there is one left

// scan for a rock to initialize
   for (int index=0; index<MAX_ROCKS3; index++)
       {
       // is this rock available?
       if (rocks3[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks3[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks3[index], xv,yv);
    
          // set position
          Set_Pos_BOB(&rocks3[index], x,y);
    
          // set size of rock
          rocks3[index].varsI[0] = size;
          rocks3[index].varsI[1] = rock_sizes[rocks3[index].varsI[0]];
          
          // set strength of rock
          switch(rocks3[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks3[index].varsI[2] = 100+rand()%100;
                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks3[index].varsI[2] = 40 + rand()%30;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks3[index].varsI[2] = 10;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks3[index].state = ROCK_STATE_ON;
          
          return;

          } // end if
       } // end for index
} // end Start_Rock3

///////////////////////////////////////////////////////////

void Draw_Rocks3(void)
{
// this function draws all the rocks2 

for (int index=0; index<MAX_ROCKS3; index++)
    {
    // test if rocks2 pulse is in flight
    if (rocks3[index].state == ROCK_STATE_ON)
        {

        if (rocks3[index].varsI[0]!=ROCK_LARGE)
            {
            // draw the rock scaled
            Draw_Scaled_BOB(&rocks3[index],
                            rocks3[index].varsI[1],rocks3[index].varsI[1],
                            lpddsback);
            }
        else // draw normal
			Draw_BOB(&rocks3[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&rocks3[index]);

        } // end if
    } // end for index
} // end Draw_Rocks3


///////////////////////////////////////////////////////////

// End Level 3
///////////////////////////////////////////////////////////

void Delete_Rocks(void)
{
// this function simply deletes all memory and surfaces
// related to the rocks pulses

for (int index=0; index<MAX_ROCKS; index++)
    Destroy_BOB(&rocks[index]);

} // end Delete_Rocks

///////////////////////////////////////////////////////////

void Move_Rocks(void)
{
	int shade;
   // this function moves all the rocks pulses and checks for
   // collision with the rocks

   for (int index=0; index<MAX_ROCKS; index++)
   {
      // test if rocks pulse is in flight
      if (rocks[index].state == ROCK_STATE_ON)
      {
         // move the rock
         Move_BOB(&rocks[index]);

         // add player motion in
         //if (moving_up)
         //    rocks[index].y+=(rocks[index].yv>>1);

         // test if rock is off screen
         if (rocks[index].y > SCREEN_HEIGHT)
         {
            // kill rock and put back on available list
            rocks[index].state = ROCK_STATE_OFF;
         } // end if

		// test for collision with the player
		// if the monsters touch the player, they cause damage to his health
        //if (Collision_Test(skelaton.x, skelaton.y, 
        //               skelaton.width, skelaton.height,
        //               rocks[index].x, rocks[index].y,
        //               rocks[index].varsI[1], rocks[index].varsI[1]))
	
		if (Collision_Test(skelaton.x, skelaton.y, 
                      skelaton.width, skelaton.height,
                      rocks[index].x, rocks[index].y,
                      rocks[index].x, rocks[index].y))

        {
			//player_hit++;
			rocks[index].state = ROCK_STATE_OFF;
			Monsters_Alive_Nbr = Monsters_Alive_Nbr + 3;
			Draw_Text_GDI("3 BABY MONSTERS ARE BORN",200,300,RGB(255,0,0),lpddsback);
			Sleep(5);
			Player_Health -= 10;
		}// end if
	  } // end if
   } // end for index

// now test if it's time to add a new rock to the list
if ((rand()%6)==3)
   {
   // scan for a rock to initialize
   for (index=0; index<MAX_ROCKS; index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          //Set_Vel_BOB(&rocks[index], -4+rand()%8, 4+rand()%12);
		  Set_Vel_BOB(&rocks[index], -1+rand()%5, 1+rand()%9);
    
          // set position
          Set_Pos_BOB(&rocks[index], rand()%SCREEN_WIDTH, -128);
    
          // set size of rock
          rocks[index].varsI[0] = rand()%3;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;
                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks[index].state = ROCK_STATE_ON;
          
          return;

          } // end if
       } // end for index
   } // end if
} // end Move_Rocks

///////////////////////////////////////////////////////////

void Start_Rock(int x, int y, int size,int xv, int yv)
{
// this functions starts a rock up with the sent parms, considering
// there is one left

// scan for a rock to initialize
   for (int index=0; index<MAX_ROCKS; index++)
       {
       // is this rock available?
       if (rocks[index].state == ROCK_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&rocks[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&rocks[index], xv,yv);
    
          // set position
          Set_Pos_BOB(&rocks[index], x,y);
    
          // set size of rock
          rocks[index].varsI[0] = size;
          rocks[index].varsI[1] = rock_sizes[rocks[index].varsI[0]];
          
          // set strength of rock
          switch(rocks[index].varsI[0])
                {
                case ROCK_LARGE:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 100+rand()%100;
                     } break;
 
                case ROCK_MEDIUM:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 40 + rand()%30;
                     } break;     

                case ROCK_SMALL:
                     {
                     // set hardness of rock
                     rocks[index].varsI[2] = 10;
                     } break;

                default:break;

                } // end switch

          // turn rock on
          rocks[index].state = ROCK_STATE_ON;
          
          // later
          return;

          } // end if
       } // end for index
} // end Start_Rock

///////////////////////////////////////////////////////////

void Draw_Rocks(void)
{
// this function draws all the rocks 

for (int index=0; index<MAX_ROCKS; index++)
    {
    // test if rocks pulse is in flight
    if (rocks[index].state == ROCK_STATE_ON)
        {

        if (rocks[index].varsI[0]!=ROCK_LARGE)
            {
            // draw the rock scaled
            Draw_Scaled_BOB(&rocks[index],
                            rocks[index].varsI[1],rocks[index].varsI[1],
                            lpddsback);
            }
        else // draw normal
           Draw_BOB(&rocks[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&rocks[index]);

        } // end if

    } // end for index

} // end Draw_Rocks

///////////////////////////////////////////////////////////
//start powerups

void Init_Powerups(void)
{
// this function initializes and loads all the powerups 

// load the powerups imagery 
Load_Bitmap_File(&bitmap8bit, "powerups.BMP");

// create the first bob
Create_BOB(&powerups[0],0,0,45,45,20,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 20; frame++)
     Load_Frame_BOB(&powerups[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
//Set_Anim_Speed_BOB(&rocks[0],1+rand()%5);
Set_Anim_Speed_BOB(&powerups[0],1);
//Set_Vel_BOB(&rocks[0], -4+rand()%8, 4+rand()%4);
Set_Vel_BOB(&powerups[0], 1, 1);
Set_Pos_BOB(&powerups[0], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);

// set size of rock
//rocks[0].varsI[0] = rand()%3;
//rocks[0].varsI[0] = rand()%3;
//rocks[0].varsI[1] = rock_sizes[rocks[0].varsI[0]];

// set state to off
powerups[0].state = POWERUP_STATE_OFF;

// initialize all the rocks by copying initialization from the first one
for (int rock=1; rock<MAX_POWERUPS; rock++)
    {
    memcpy(&powerups[rock], &powerups[0], sizeof(BOB));

    // set animation rate
    //Set_Anim_Speed_BOB(&rocks[rock],1+rand()%5);
	Set_Anim_Speed_BOB(&powerups[rock],1);
 
    // set velocity
    //Set_Vel_BOB(&rocks[rock], -1+rand()%3, 2+rand()%2);
	Set_Vel_BOB(&powerups[rock], 0, 1);
    
    // set position
    Set_Pos_BOB(&powerups[rock], rand()%SCREEN_WIDTH, rand()%SCREEN_HEIGHT);
    
    } // end for powerup

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Rocks

///////////////////////////////////////////////////////////

void Delete_Powerups(void)
{
// this function simply deletes all memory and surfaces
// related to the powerups pulses

for (int index=0; index<MAX_POWERUPS; index++)
    Destroy_BOB(&powerups[index]);

} // end Delete_Powerups

///////////////////////////////////////////////////////////

void Move_Powerups(void)
{
   // this function moves all the powerups pulses and checks for
   // collision with the powerups

   for (int index=0; index<MAX_POWERUPS; index++)
   {
      // test if powerups pulse is in flight
      if (powerups[index].state == POWERUP_STATE_ON)
      {
         // move the powerup
         Move_BOB(&powerups[index]);

         // test if powerup is off screen
         if (powerups[index].y > SCREEN_HEIGHT)
         {
            // kill powerup and put back on available list
            powerups[index].state = POWERUP_STATE_OFF;
         } // end if
        
       } // end if

	   // test for collision with the player
	   // if the powerups touch the player, they add energy to the player
	   if (powerups[index].state == POWERUP_STATE_ON)
	   {
		  if (Collision_Test(skelaton.x, skelaton.y, 
              skelaton.width, skelaton.height,
              powerups[index].x, powerups[index].y,
              powerups[index].varsI[1], powerups[index].varsI[1]))
		  {      
		     // set energy flag which means energy is acquired by the player from the powerup
			 Energy_Flag = 1;
		  
		     if (Level2_Flag == 0)
			 {
			    // add energy to player
			    powerups[index].state = POWERUP_STATE_OFF;
			    Player_Energy += 25;
			 }
		     else if ((Level2_Flag == 1) && (Level3_Flag == 0))
			 {
			    // add energy to player
			    powerups[index].state = POWERUP_STATE_OFF;
			    Player_Energy += 15;
			 }
		     else if (Level3_Flag == 1)
			 {
			    // add energy to player
			    powerups[index].state = POWERUP_STATE_OFF;
			    Player_Energy += 10;
			 }
		  }//end if 
	   }//end if

	   // GET RID OF THIS CONDITION BECAUSE WHEN MONSTERS TOUCH POWERUPS
	   // NOTHING HAPPENS.  ONLY WHEN PLAYER TOUCHES MONSTERS
	   // SO NO NEED FOR TWO ROCKS 
	   // test for collision with the monster
	   // if the powerups touch the monster, they add energy to the monster
	   for (int rock=0; rock<MAX_ROCKS; rock++)
       {
          if (rocks[rock].state==ROCK_STATE_ON)
          {
             // test for collision 
             if (Collision_Test(plasma[index].x, plasma[index].y, 
                                   plasma[index].width, plasma[index].height,
                                   rocks[rock].x, rocks[rock].y,
                                   rocks[rock].varsI[1], rocks[rock].varsI[1]))
             {
                // kill powerup
                powerups[index].state = POWERUP_STATE_OFF;

			 }//end if collision
		  }//end if
	   }//end for rock
} // end for index

// now test if it's time to add a new powerup to the list
if ((rand()%6)==3)
   {
   // scan for a powerup to initialize
   for (index=0; index<MAX_POWERUPS; index++)
       {
       // is this rock available?
       if (powerups[index].state == POWERUP_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&powerups[index],1+rand()%5);
    
          // set velocity
          //Set_Vel_BOB(&rocks[index], -4+rand()%8, 4+rand()%12);
		  Set_Vel_BOB(&powerups[index], -1+rand()%5, 1+rand()%9);
    
          // set position
          Set_Pos_BOB(&powerups[index], rand()%SCREEN_WIDTH, -128);

          // turn powerup on
          powerups[index].state = POWERUP_STATE_ON;
          
          return;

          } // end if
       } // end for index
   } // end if
} // end Move_Powerups

///////////////////////////////////////////////////////////

void Start_Powerup(int x, int y, int size,int xv, int yv)
{
// this functions starts a powerup up with the sent parms, considering
// there is one left

// scan for a powerup to initialize
   for (int index=0; index<MAX_POWERUPS; index++)
       {
       // is this powerup available?
       if (powerups[index].state == POWERUP_STATE_OFF)
          {
          // set animation rate
          Set_Anim_Speed_BOB(&powerups[index],1+rand()%5);
    
          // set velocity
          Set_Vel_BOB(&powerups[index], xv,yv);
    
          // set position
          Set_Pos_BOB(&powerups[index], x,y);

          // turn powerup on
          powerups[index].state = POWERUP_STATE_ON;
          
          return;

          } // end if
       } // end for index
} // end Start_Powerup

///////////////////////////////////////////////////////////

void Draw_Powerups(void)
{
// this function draws all the Powerups 

for (int index=0; index<MAX_POWERUPS; index++)
    {
    // test if powerups pulse is in flight
    if (powerups[index].state == POWERUP_STATE_ON)
        {
		// draw normal
        Draw_BOB(&powerups[index],lpddsback);
         
        // animate the pulse
        Animate_BOB(&powerups[index]);

        } // end if
    } // end for index
}// end Draw_Powerups

//end powerups
///////////////////////////////////////////////////////////

void Init_Bursts(void)
{
// this function initializes and loads all the bursts 

// load the bursts imagery 
Load_Bitmap_File(&bitmap8bit, "MonsterExplosion.BMP");

// create the first bob
Create_BOB(&bursts[0],0,0,42,36,24,
            BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME,
            DDSCAPS_SYSTEMMEMORY);

// load animation frames
for (int frame=0; frame < 24; frame++)
     Load_Frame_BOB(&bursts[0],&bitmap8bit,frame,frame%6,frame/6,BITMAP_EXTRACT_MODE_CELL);  

// set animation rate
Set_Anim_Speed_BOB(&bursts[0],1);

// set size of burst
bursts[0].varsI[0] = bursts[0].width;
bursts[0].varsI[1] = bursts[0].height;

// set state to off
bursts[0].state = BURST_STATE_OFF;

for (int burst=1; burst<MAX_BURSTS; burst++)
    {
    memcpy(&bursts[burst], &bursts[0], sizeof(BOB));
    } // end for burst

// unload data infile
Unload_Bitmap_File(&bitmap8bit);

} // end Init_Bursts

///////////////////////////////////////////////////////////

void Delete_Bursts(void)
{
// this function simply deletes all memory and surfaces
// related to the bursts pulses

for (int index=0; index<MAX_BURSTS; index++)
    Destroy_BOB(&bursts[index]);

} // end Delete_Bursts

///////////////////////////////////////////////////////////

void Move_Bursts(void)
{
// this function moves all the bursts 

for (int index=0; index<MAX_BURSTS; index++)
    {
    // test if bursts pulse is in moving
    if (bursts[index].state == BURST_STATE_ON)
       {
       // move the burst
       Move_BOB(&bursts[index]);

        // test if burst is off screen or done with animation
        if ((bursts[index].y > SCREEN_HEIGHT) || bursts[index].curr_frame >= bursts[index].num_frames-1) 
           {
           // kill burst and put back on available list
           bursts[index].state = BURST_STATE_OFF;
           } // end if
        
        } // end if
    } // end for index
} // end Move_Bursts

///////////////////////////////////////////////////////////

void Draw_Bursts(void)
{
// this function draws all the bursts 
for (int index=0; index<MAX_BURSTS; index++)
    {
    // test if bursts pulse is in flight
    if (bursts[index].state == BURST_STATE_ON)
        {
        if (bursts[index].varsI[0]!=bursts[index].width || 
            bursts[index].varsI[1]!=bursts[index].height)
            {
            // draw the burst scaled
            Draw_Scaled_BOB(&bursts[index],
                            bursts[index].varsI[0],bursts[index].varsI[1],
                            lpddsback);
            }
        else // draw normal
           Draw_BOB(&bursts[index],lpddsback);
         
        // animate the explosion
        Animate_BOB(&bursts[index]);

        } // end if
    } // end for index
} // end Draw_Bursts

///////////////////////////////////////////////////////////

void Start_Burst(int x, int y, int width, int height, int xv,int yv)
{
// this function starts a burst up

// now test if it's time to add a new burst to the list

// scan for a burst to initialize
for (int index=0; index<MAX_BURSTS; index++)
    {
    // is this burst available?
    if (bursts[index].state == BURST_STATE_OFF)
       {
       // set animation rate
       Set_Anim_Speed_BOB(&bursts[index],1);
       bursts[index].curr_frame = 0;
    
       // set velocity
       Set_Vel_BOB(&bursts[index], xv,yv);
    
       // set position
       Set_Pos_BOB(&bursts[index], x,y);
    
       // set size of burst
       bursts[index].varsI[0] = width;
       bursts[index].varsI[1] = height;

       // turn burst on
       bursts[index].state = BURST_STATE_ON;

       // start sound up
       for (int sound_index=0; sound_index < MAX_EXPL_SOUNDS; sound_index++)
           {
           // test if this sound is playing
           if (DSound_Status_Sound(expl_ids[sound_index])==0)
              {
              DSound_Play(expl_ids[sound_index]);
              break;
              } // end if        
           } // end for sound_index       

       return;

       } // end if
    } // end for index
} // end Start_Burst

///////////////////////////////////////////////////////////

void Draw_Info(void)
{
int shade; // used as looping var

// this function draws all the information at the top of the screen

// Show Health points in the top left corner
sprintf(buffer,"HEALTH %d",Player_Health);
Draw_Text_GDI(buffer,10,10,RGB(0,255,0),lpddsback);

// Show Energy points in the top right corner
sprintf(buffer,"ENERGY %d",Player_Energy);
Draw_Text_GDI(buffer,520,10,RGB(0,255,0),lpddsback);

// Show Number of lived monsters in the lower right corner
sprintf(buffer,"ALIVE MONSTERS %d",Monsters_Alive_Nbr);
Draw_Text_GDI(buffer,475,450,RGB(0,255,0),lpddsback);

// Show Number of dead monsters in the lower left corner
sprintf(buffer,"DEAD MONSTERS %d",Monsters_Dead_Nbr);
Draw_Text_GDI(buffer,10,450,RGB(0,255,0),lpddsback);

if (Level2_Flag == 0)
{
	Draw_Text_GDI("LEVEL 1",270,10,RGB(0,255,0),lpddsback);
}
else if ((Level2_Flag == 1) && (Level3_Flag == 0))
{
	Draw_Text_GDI("LEVEL 2",270,10,RGB(255,0,0),lpddsback);
}//end if
else if (Level3_Flag == 1)
{
	Draw_Text_GDI("LEVEL 3",270,10,RGB(0,0,255),lpddsback);
}//end else if


// draw new baby born message
//if (L1_Birth_Flag == 0)
//{
//	for (shade=0; shade < 256; shade++)
//    {
    // draw text in shades
//	Draw_Text_GDI("3 new baby monsters are born",320,450,RGB(0,255,0),lpddsback);
//	Sleep(2);
    
    // is user trying to bail
//    if (KEY_DOWN(VK_ESCAPE))
//        return;

//    } // end for shade

//	Sleep(1000);
//}

// draw energy acquired message
if (Energy_Flag)
{
	for (shade=0; shade < 256; shade++)
    {
    // draw text in shades
	strcpy(buffer,"You Have Gained Energy");
	Draw_Text_GDI(buffer,320-8*strlen(buffer),500,RGB(0,255,0),lpddsback);
	Sleep(2);
    
    // is user trying to bail
    if (KEY_DOWN(VK_ESCAPE))
        return;

    } // end for shade
Sleep(1000);
	
}

} // end Draw_Info

///////////////////////////////////////////////////////////

void Do_Intro(void)
{
// the worlds simples intro

int shade; // used as looping var

for (shade=0; shade < 256; shade++)
    {
    // draw text in shades
    Draw_Text_GDI("El-Assaad Entertaining Products present...",320-4*strlen("El-Assaad Entertaining Products present"),200,
                   RGB(shade,shade,shade),lpddsprimary);
    Draw_Text_GDI("Genetic Monsters",320-4*strlen("Genetic Monsters"),230,
                   RGB(shade,shade,shade),lpddsprimary);
    Draw_Text_GDI("Version I",320-4*strlen("Version I"),260,
                   RGB(shade,shade,shade),lpddsprimary);

    Sleep(2);
    
    // is user trying to bail
    if (KEY_DOWN(VK_ESCAPE))
        return;

    } // end for shade

Sleep(1000);

for (shade=255; shade >=0; shade--)
    {
    // draw text in shades
    Draw_Text_GDI("El-Assaad Entertaining Products present...",320-4*strlen("El-Assaad Entertaining Products present"),200,
                   RGB(shade,0,0),lpddsprimary);
    Draw_Text_GDI("Genetic Monsters",320-4*strlen("Genetic Monsters"),230,
                   RGB(shade,0,0),lpddsprimary);
    Draw_Text_GDI("Version I",320-4*strlen("Version I"),260,
                   RGB(shade,0,0),lpddsprimary);

    Sleep(2);
    
    // is user trying to bail
    if (KEY_DOWN(VK_ESCAPE))
        return;

    } // end for shade

Sleep(1000);

// load first bitmap (introduction)
// load bitmap into primary buffer
Load_Bitmap_File(&bitmap8bit, "BackIntro.BMP");
Create_Bitmap(&background_bmp,0,0,640,480);
Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
DD_Lock_Primary_Surface();
Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
DD_Unlock_Primary_Surface();
Unload_Bitmap_File(&bitmap8bit);
Destroy_Bitmap(&background_bmp);

Sleep(3000);

// load last bitmap (end)
// load bitmap into primary buffer
Load_Bitmap_File(&bitmap8bit, "BackHelp.BMP");
Create_Bitmap(&background_bmp,0,0,640,480);
Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
DD_Lock_Primary_Surface();
Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
DD_Unlock_Primary_Surface();
Unload_Bitmap_File(&bitmap8bit);
Destroy_Bitmap(&background_bmp);

Sleep(14000);

// load last bitmap (end)
// load bitmap into primary buffer
Load_Bitmap_File(&bitmap8bit, "BackDirections.BMP");
Create_Bitmap(&background_bmp,0,0,640,480);
Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
DD_Lock_Primary_Surface();
Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
DD_Unlock_Primary_Surface();
Unload_Bitmap_File(&bitmap8bit);
Destroy_Bitmap(&background_bmp);

Sleep(8000);


} // end Do_Intro

///////////////////////////////////////////////////////////

int Load_Sound_Music(void)
{
// this function loads all the sounds and music

// load in intro music
if ((intro_music_id = DSound_Load_WAV("OUTSOUND/INTRO.WAV"))==-1)
   return(0);

// load in intro music
if ((applause_id = DSound_Load_WAV("OUTSOUND/Applause.WAV"))==-1)
   return(0);
    
// load the main music
if ((main_music_id = DSound_Load_WAV("OUTSOUND/STARSNG.WAV"))==-1)
   return(0);

// load get ready
if ((ready_id = DSound_Load_WAV("OUTSOUND/ENTERING1.WAV"))==-1)
   return(0);

// load engines
if ((engines_id = DSound_Load_WAV("OUTSOUND/ENGINES.WAV"))==-1)
   return(0);

// load scream
if ((scream_id = DSound_Load_WAV("OUTSOUND/BREAKUP.WAV"))==-1)
   return(0);

// load game over
if ((game_over_id = DSound_Load_WAV("OUTSOUND/GAMEOVER.WAV"))==-1)
   return(0);

// load mine powerup
if ((mine_powerup_id = DSound_Load_WAV("OUTSOUND/MINEPOWER1.WAV"))==-1)
   return(0);

if ((deactivate_id = DSound_Load_WAV("OUTSOUND/DEACTIVATE1.WAV"))==-1)
   return(0);

if ((station_throb_id = DSound_Load_WAV("OUTSOUND/STATIONTHROB2.WAV"))==-1)
   return(0);


if ((beep0_id = DSound_Load_WAV("OUTSOUND/BEEP3.WAV"))==-1)
   return(0);

if ((beep1_id = DSound_Load_WAV("OUTSOUND/BEEP1.WAV"))==-1)
   return(0);

if ((correct_id = DSound_Load_WAV("OUTSOUND/CORRECT1.WAV"))==-1)
   return(0);

// load the explosions sounds
if ((station_blow_id = DSound_Load_WAV("OUTSOUND/STATIONBLOW.WAV"))==-1)
   return(0);


// these are the two different source masters
if ((expl_ids[0] = DSound_Load_WAV("OUTSOUND/EXPL1.WAV"))==-1)
    return(0);

if ((expl_ids[1] = DSound_Load_WAV("OUTSOUND/EXPL2.WAV"))==-1)
    return(0);

// now make copies
for (int index=2; index < MAX_EXPL_SOUNDS; index++)
    {
    // replicate sound
    expl_ids[index] = DSound_Replicate_Sound(expl_ids[rand()%2]);  

    } // end for index

// load the plasma weapons sounds
if ((fire_ids[0] = DSound_Load_WAV("OUTSOUND/PULSE.WAV"))==-1)
    return(0);

// now make copies
for (index=1; index < MAX_FIRE_SOUNDS; index++)
    {
    // replicate sound
    fire_ids[index] = DSound_Replicate_Sound(fire_ids[0]);  
    } // end for index

// return sucess
return(1);

} // end Load_Sound_Music

///////////////////////////////////////////////////////////////////


// screen is the number of screen sequences in that level
int Switch_Level(int screen)
{

char background[80]; // used to build up files names

// Gets rid of the background
Destroy_Bitmap(&reactor);
	
// Loads the correct background according to the screen
sprintf(background,"backgnd%d.BMP",screen);

// load the background 2
//Load_Bitmap_File(&bitmap8bit, "1BACK2.BMP");
Load_Bitmap_File(&bitmap8bit, background);

// set the palette to background image palette
Set_Palette(bitmap8bit.palette);

// create and load the reactor bitmap image
Create_Bitmap(&reactor, 0,0, 640, 480);
Load_Image_Bitmap(&reactor,&bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
Unload_Bitmap_File(&bitmap8bit);

// Puts the character back to the bottom or top of the screen
if (screen == 1)
	skelaton.y = 400;
else {
	if (skelaton.y < 200)
		skelaton.y+=400;
	else
		skelaton.y-=400;
}
	
return(1);

}

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

// delete all the explosions
Delete_Bursts();

// kill the reactor
Destroy_Bitmap(&reactor);

// kill skelaton
Destroy_BOB(&skelaton);

if (Level2_Flag == 0)
{
	// the delete all the rocks
	Delete_Rocks();
}
else if ((Level2_Flag == 1) && (Level3_Flag == 0))
{
	// the delete all the rocks
	Delete_Rocks2();
}
else if (Level3_Flag == 1)
{
	// the delete all the rocks
	Delete_Rocks3();
}

// delete the powerups
Delete_Powerups();

// delete all the plasma pulses
Delete_Plasma();

// release keyboard
lpdikey->Unacquire();
lpdikey->Release();
lpdi->Release();

// shutdonw directdraw
DD_Shutdown();

// return success
return(1);
} // end Game_Shutdown

///////////////////////////////////////////////////////////

int Game_Main(void *parms)
{
// this is the workhorse of your game it will be called
// continuously in real-time this is like main() in C
// all the calls for your game go here!

int          index;             // looping var
int          dx,dy;             // general deltas used in collision detection
 
static int   player_moving = 0; // tracks player motion
//static int   Birth_Flag = 0;    // tracks monster birth

// static PALETTEENTRY glow = {0,0,0,PC_NOCOLLAPSE};  // used to animation red border

// get the device context
HDC hdc = GetDC(main_window_handle);

// check of user is trying to exit
if (KEY_DOWN(VK_ESCAPE))
    PostMessage(main_window_handle, WM_DESTROY,0,0);

// start the timing clock
Start_Clock();

// clear the drawing surface
DD_Fill_Surface(lpddsback, 0);

// lock the back buffer
DD_Lock_Back_Surface();

// draw the background reactor image
Draw_Bitmap(&reactor, back_buffer, back_lpitch, 0);

// unlock the back buffer
DD_Unlock_Back_Surface();

// get player input

// get the keyboard data
lpdikey->GetDeviceState(256, (LPVOID)keyboard_state);

// reset motion flag
player_moving = 0;

// reset energy flag
Energy_Flag = 0;

// test direction of motion, this is a good example of testing the keyboard
// although the code could be optimized this is more educational

if (keyboard_state[DIK_RIGHT] && keyboard_state[DIK_UP]) 
   {
   // move skelaton
   skelaton.x+=2;
   skelaton.y-=2;
   dx=2; dy=-2;
   plasma_x=16; plasma_y=-16;

   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_NEAST)
      Set_Animation_BOB(&skelaton,SKELATON_NEAST);

   } // end if
else
if (keyboard_state[DIK_LEFT] && keyboard_state[DIK_UP]) 
   {
   // move skelaton
   skelaton.x-=2;
   skelaton.y-=2;
   dx=-2; dy=-2;
   plasma_x=-16; plasma_y=-16;

   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_NWEST)
      Set_Animation_BOB(&skelaton,SKELATON_NWEST);

   } // end if
else
if (keyboard_state[DIK_LEFT] && keyboard_state[DIK_DOWN]) 
   {
   // move skelaton
   skelaton.x-=2;
   skelaton.y+=2;
   dx=-2; dy=2;
   plasma_x=-16; plasma_y=16;

   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_SWEST)
      Set_Animation_BOB(&skelaton,SKELATON_SWEST);

   } // end if
else
if (keyboard_state[DIK_RIGHT] && keyboard_state[DIK_DOWN]) 
   {
   // move skelaton
   skelaton.x+=2;
   skelaton.y+=2;
   dx=2; dy=2;
   plasma_x=16; plasma_y=16;

   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_SEAST)
      Set_Animation_BOB(&skelaton,SKELATON_SEAST);

   } // end if
else
if (keyboard_state[DIK_RIGHT]) 
   {
   // move skelaton
   skelaton.x+=2;
   dx=2; dy=0;
   plasma_x=16; plasma_y=0;

   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_EAST)
      Set_Animation_BOB(&skelaton,SKELATON_EAST);

   } // end if
else
if (keyboard_state[DIK_LEFT])  
   {
   // move skelaton
   skelaton.x-=2;
   dx=-2; dy=0; 
   plasma_x=-16; plasma_y=0;
   
   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_WEST)
      Set_Animation_BOB(&skelaton,SKELATON_WEST);

   } // end if
else
if (keyboard_state[DIK_UP])    
   {
   // move skelaton
   skelaton.y-=2;
   dx=0; dy=-2;
   plasma_x=0; plasma_y=-16;
   
   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_NORTH)
      Set_Animation_BOB(&skelaton,SKELATON_NORTH);

   } // end if
else
if (keyboard_state[DIK_DOWN])  
   {
   // move skelaton
   skelaton.y+=2;
   dx=0; dy=+2;
   plasma_x=0; plasma_y=16;

   // set motion flag
   player_moving = 1;

   // check animation needs to change
   if (skelaton.curr_animation != SKELATON_SOUTH)
      Set_Animation_BOB(&skelaton,SKELATON_SOUTH);

   } // end if

// test if player is firing
if (KEY_DOWN(VK_CONTROL) || KEY_DOWN(VK_SPACE))
   Fire_Plasma(skelaton.x+28-8,skelaton.y-8,plasma_x, plasma_y);

// Transfer Energy to Health if player press F8
if (KEY_DOWN(VK_F8))  
{
	if (Player_Energy >= 50)
	{
		Player_Health += 50;
		Player_Energy -= 50;
	}
}

// only animate if player is moving
if (player_moving)
{
   // animate skelaton
   Animate_BOB(&skelaton);

   // see if skelaton hit a wall
   
   // lock surface, so we can scan it
   DD_Lock_Back_Surface();
   
   // call the color scanner with 29, the color of the glowing wall
   // try to center the scan in the center of the object to make it 
   // more realistic
   if (Color_Scan(skelaton.x+16, skelaton.y+16,
                  skelaton.x+skelaton.width-16, skelaton.y+skelaton.height-16,                                    
                  30, back_buffer,back_lpitch))
   {
      // back the skelaton up along its last trajectory
      skelaton.x-=dx;
      skelaton.y-=dy;
   }// end if
   
   // This will switch screens forward if the color is encountered
   if (Color_Scan(skelaton.x+16, skelaton.y+16,
                  skelaton.x+skelaton.width-16, skelaton.y+skelaton.height-16,                                    
                  59, back_buffer,back_lpitch))
   {
	  // Depending on location of character, will go forward or backward
	 
	  if ( skelaton.y < 200 )
		screen_seq++;
	  else
		screen_seq--;

	  if (screen_seq == 7)
	  {
	     // load bitmap into primary buffer
		 Load_Bitmap_File(&bitmap8bit, "backgnd1.bmp");
		 Create_Bitmap(&background_bmp,0,0,640,480);
		 Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
		 DD_Lock_Primary_Surface();
		 Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
		 DD_Unlock_Primary_Surface();
		 Unload_Bitmap_File(&bitmap8bit);
		 Destroy_Bitmap(&background_bmp);

		 Sleep(15000);
		 Game_Shutdown();
	  }
	  else 
	  {
	     // Calls this function, which loads a new background
         Switch_Level(screen_seq);
	  }
   }// end if

   // if the player has been hit too many times, then starts over.
   if (player_hit > player_limit)
   {
	  player_hit=0;
	  screen_seq=1;
	  Switch_Level(screen_seq);
   }

   // done, so unlock
   DD_Unlock_Back_Surface();

}// end if



// draw the skelaton
Draw_BOB(&skelaton, lpddsback);

// draw the score and ships left
Draw_Info();

// move the plasma
Move_Plasma();

if (Level2_Flag == 0)
{
	// move the asteroids
	Move_Rocks();	
}
else if ((Level2_Flag == 1) && (Level3_Flag == 0))
{
	// move the asteroids
	Move_Rocks2();
}//end if
else if (Level3_Flag == 1)
{
	// move the asteroids
	Move_Rocks3();
}//end if


// move the powerups
Move_Powerups();

// move the explosions
Move_Bursts();

if (Level2_Flag == 0)
{
	// draw the rocks
	Draw_Rocks();	
}
else if ((Level2_Flag == 1) && (Level3_Flag == 0))
{
	// draw the rocks
	Draw_Rocks2();
}//end if
else if (Level3_Flag == 1)
{
	// draw the rocks
	Draw_Rocks3();
}//end if


// draw the powerups
Draw_Powerups();

// draw the plasma
Draw_Plasma();

// draw explosions last
Draw_Bursts();

// draw the statistics
//Draw_Info();

// flip the surfaces
DD_Flip();

// sync to 3o fps
Wait_Clock(30);

// End game by winning if the Player killed all monsters
// Player switches to level2
if ((Monsters_Alive_Nbr <= 0) && (Level2_Flag == 0))
{
   // load bitmap into primary buffer
   Load_Bitmap_File(&bitmap8bit, "BackMain.bmp");
   Create_Bitmap(&background_bmp,0,0,640,480);
   Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
   DD_Lock_Primary_Surface();
   Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
   DD_Unlock_Primary_Surface();
   Unload_Bitmap_File(&bitmap8bit);
   Destroy_Bitmap(&background_bmp);

   // reset Level 2 flag
   Level2_Flag = 1;

   // reset all statistics in the game
   // init rocks
   Init_Rocks2();
   Monsters_Alive_Nbr = 40;  // SHOULD BE 40
   Monsters_Dead_Nbr = 0;
   Player_Health = 500;
   Player_Energy = 500;

   // play game over sound
   //DSound_Play(game_over_id);

   // shutdown the game
   //Sleep(4000);
   //Game_Shutdown();

}
else if ((Monsters_Alive_Nbr <= 0) && (Level2_Flag == 1) && (Level3_Flag == 0))
{
   // load bitmap into primary buffer
   Load_Bitmap_File(&bitmap8bit, "BackMain.bmp");
   Create_Bitmap(&background_bmp,0,0,640,480);
   Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
   DD_Lock_Primary_Surface();
   Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
   DD_Unlock_Primary_Surface();
   Unload_Bitmap_File(&bitmap8bit);
   Destroy_Bitmap(&background_bmp);

   // reset Level 2 flag
   Level3_Flag = 1;

   // reset all statistics in the game
   // init rocks
   Init_Rocks3();
   Monsters_Alive_Nbr = 60; // SHOULD BE 60
   Monsters_Dead_Nbr = 0;
   Player_Health = 500;
   Player_Energy = 500;

   // play game over sound
   //DSound_Play(game_over_id);

   // shutdown the game
   //Sleep(4000);
   //Game_Shutdown();

}
else if ((Monsters_Alive_Nbr <= 0) && (Level2_Flag == 1) && (Level3_Flag == 1))
{
   // load bitmap into primary buffer
   Load_Bitmap_File(&bitmap8bit, "BackWin.bmp");
   Create_Bitmap(&background_bmp,0,0,640,480);
   Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
   DD_Lock_Primary_Surface();
   Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
   DD_Unlock_Primary_Surface();
   Unload_Bitmap_File(&bitmap8bit);
   Destroy_Bitmap(&background_bmp);

   // play game over sound
   DSound_Play(applause_id);

   // shutdown the game
   Sleep(6000);
   Game_Shutdown();

}


// End game by loosing if the Player Health drops to ZERO
if (Player_Health <= 0)
{
   // load bitmap into primary buffer
   Load_Bitmap_File(&bitmap8bit, "BackLoose.bmp");
   Create_Bitmap(&background_bmp,0,0,640,480);
   Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_CELL);
   DD_Lock_Primary_Surface();
   Draw_Bitmap(&background_bmp,primary_buffer, primary_lpitch, 0);
   DD_Unlock_Primary_Surface();
   Unload_Bitmap_File(&bitmap8bit);
   Destroy_Bitmap(&background_bmp);
   
   // play game over sound
   DSound_Play(game_over_id);

   Sleep(6000);
   Game_Shutdown();
}

// check of user is trying to exit
if (KEY_DOWN(VK_ESCAPE))
    PostMessage(main_window_handle, WM_DESTROY,0,0);

// release the device context back to windows
ReleaseDC(main_window_handle, hdc);


// return success
return(1);

} // end Game_Main

//////////////////////////////////////////////////////////